/*

*/

import tokenizer.*;
import tree.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;

/**
   This program implements an "interpreter" for the
   hw2 expression language.
   <p>
   This program provides a REPL (Read Eval Print Loop)
   for the language.
*/
public class Hw2_demo
{
   public static void main(String[] args) throws IOException
   {
      runPrompt();
   }


   private static void runPrompt() throws IOException
   {
      final InputStreamReader input = new InputStreamReader(System.in);
      final BufferedReader reader = new BufferedReader(input);

      System.out.println("Hw2_demo.");

      // repl (read, eval, print loop)
      for (int i = 1;; i++)
      {
         System.out.print(i + ":> ");

         // Read.
         final String source = reader.readLine();

         try
         {
            Tokenizer tokens = new Tokenizer(source);
            if ( ! tokens.hasToken() )
            {
               continue; // skip over empty string
            }

            // Eval.
            final int r = Hw2_Parse2Value.getE(tokens);
            if ( tokens.hasToken() )  // there shouldn't be any more tokens
               throw new ParseError("unexpected input: "
                                    + tokens.peek().lexeme
                                    + " at position " + tokens.peek().position
                                    + "\n");

            // Print the result.
            System.out.println(r);

            tokens = new Tokenizer(source);  // rewind
            final Tree<Token> ast = Hw2_Parse2AST.getE(tokens);

            tokens = new Tokenizer(source);  // rewind
            final Tree<String> parseTree = Hw2_Parse2Syntax.getE(tokens);

            // Create dot and png files from the trees.
            String baseName = String.format("Exp_%02d_AST", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            baseName = String.format("Exp_%02d_Parse", i);
            Tree2dot.tree2dot(parseTree, baseName);
            Tree2png.tree2png(baseName);
         }
         catch (TokenizeError e)
         {
            System.out.println(e);
         }
         catch (ParseError e)
         {
            System.out.println(e);
         }
      }
   }
}
