/*

*/

package tree;

/**
   The prettyPrint() method takes a {@link Tree}
   and converts it into a well formatted string.
*/
public class PrettyPrinter
{
   public static final String INDENTATION = "  ";

   /**
      Pretty print a {@link Tree} with each sub tree
      indented by a certain amount.

      @param <T>   The element type for the {@link Tree}
      @param tree  {@link Tree} to be pretty printed
      @return the {@link String} representation of the prettyprinted {@link Tree}
   */
   public static <T> String prettyPrint(final Tree<T> tree)
   {
      return prettyPrint(tree, "");
   }


   /**
      This is a recursive helper function for the prettyPrint method.
      <p>
      This method pretty prints a {@link Tree} with a given amount
      of indentation. If the pretty printer needs to prettyprint a
      sub tree, then the sub tree gets a larger amount of indentation.
      <p>
      This method is essentially a pre-order traversal of the {@link Tree}.

      @param <T>     The element type for the {@link Tree}
      @param tree    {@link Tree} to be pretty printed
      @param indent  {@link String} holding the current left margin indentation
      @return the {@link String} representation of the given {@link Tree} prettyprinted and indented
   */
   private static <T> String prettyPrint(final Tree<T> tree,
                                         final String indent)
   {
      String result = indent;

      if ( tree.isLeaf() ) // stops the recursion
      {
         result += tree.root();
      }
      else
      {
         result += "(" + tree.root();

         // recursively traverse all the sub trees
         for ( final Tree<T> subTree : tree.subTrees() )
         {
            result += "\n" + prettyPrint(subTree, indent + INDENTATION);
         }

         result += "\n" + indent + ")";
      }

      return result;
   }
}
